@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2006-2021 by
+ * Copyright (C) 2006-2025 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -36,7 +36,7 @@ public class AudibleAlertsParameter extends GlobalParameter implements Parameter
   }
 
   @Override
-  public final void set (boolean yes) {
-    setValue(new boolean[] {yes});
+  public final void set (long subparam, boolean yes) {
+    setValue(subparam, new boolean[] {yes});
   }
 }
