@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2006-2023 by
+ * Copyright (C) 2006-2025 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -30,6 +30,11 @@ public class CommandKeycodeSummaryParameter extends GlobalParameter {
     return Constants.PARAM_COMMAND_KEYCODE_SUMMARY;
   }
 
+  @Override
+  public boolean hasSubparam () {
+    return true;
+  }
+
   @Override
   public final String get (long code) {
     return asString(getValue(code));
