@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -52,6 +52,7 @@
 #include "cmd_toggle.h"
 #include "cmd_touch.h"
 
+#include "async_handle.h"
 #include "async_wait.h"
 #include "async_alarm.h"
 #include "async_event.h"
@@ -71,6 +72,7 @@
 #include "brl_utils.h"
 #include "prefs.h"
 #include "api_control.h"
+#include "options.h"
 #include "core.h"
 
 #ifdef ENABLE_SPEECH_SUPPORT
@@ -78,6 +80,12 @@
 #endif /* ENABLE_SPEECH_SUPPORT */
 
 BrailleDisplay brl;                        /* For the Braille routines */
+
+int
+haveBrailleDisplay (void) {
+  return braille->definition.code != noBraille.definition.code;
+}
+
 ScreenDescription scr;
 SessionEntry *ses = NULL;
 
@@ -250,6 +258,7 @@ handleUnhandledCommands (int command, void *data) {
 
 static int
 handleApiCommands (int command, void *data) {
+  if (isIneligibleApiCommand(command)) return 0;
   return api.handleCommand(command);
 }
 
@@ -438,7 +447,7 @@ writeBrailleCharacters (const char *mode, const wchar_t *characters, size_t leng
 
   fillStatusSeparator(textBuffer, brl.buffer);
 
-  return writeBrailleWindow(&brl, textBuffer, 0);
+  return writeBrailleWindow(&brl, textBuffer, SCQ_GOOD);
 }
 
 int
@@ -900,11 +909,10 @@ trackScreenCursor (int place) {
   ses->dcty = 0;
 
   if (isContracted) {
+    slideBrailleWindowVertically(scr.posy);
     contractedTrack = 1;
 
-    if (scr.posy != ses->winy) {
-      ses->winy = scr.posy;
-    } else if (scr.posx > ses->winx) {
+    if (scr.posx > ses->winx) {
       if (scr.posx < (ses->winx + getContractedLength(textCount))) {
         return 1;
       }
@@ -956,20 +964,12 @@ trackScreenCursor (int place) {
   return 1;
 }
 
-ScreenCharacterType
-getScreenCharacterType (const ScreenCharacter *character) {
-  if (iswspace(character->text)) return SCT_SPACE;
-  if (iswalnum(character->text)) return SCT_WORD;
-  if (wcschr(WS_C("_"), character->text)) return SCT_WORD;
-  return SCT_NONWORD;
-}
-
 int
 findFirstNonSpaceCharacter (const ScreenCharacter *characters, int count) {
   int index = 0;
 
   while (index < count) {
-    if (getScreenCharacterType(&characters[index]) != SCT_SPACE) return index;
+    if (!iswspace(characters[index].text)) return index;
     index += 1;
   }
 
@@ -981,7 +981,7 @@ findLastNonSpaceCharacter (const ScreenCharacter *characters, int count) {
   int index = count;
 
   while (index > 0)
-    if (getScreenCharacterType(&characters[--index]) != SCT_SPACE)
+    if (!iswspace(characters[--index].text))
       return index;
 
   return -1;
@@ -995,6 +995,11 @@ isAllSpaceCharacters (const ScreenCharacter *characters, int count) {
 #ifdef ENABLE_SPEECH_SUPPORT
 SpeechSynthesizer spk;
 
+int
+haveSpeechSynthesizer (void) {
+  return speech->definition.code != noSpeech.definition.code;
+}
+
 void
 trackSpeech (void) {
   int location = spk.track.speechLocation;
@@ -1008,31 +1013,42 @@ trackSpeech (void) {
 
 int
 isAutospeakActive (void) {
-  if (speech->definition.code == noSpeech.definition.code) return 0;
+  if (!haveSpeechSynthesizer()) return 0;
   if (prefs.autospeak) return 1;
-  if (braille->definition.code != noBraille.definition.code) return 0;
+  if (haveBrailleDisplay()) return 0;
   return !opt_quietIfNoBraille;
 }
 
 void
 sayScreenCharacters (const ScreenCharacter *characters, size_t count, SayOptions options) {
-  wchar_t text[count];
-  wchar_t *t = text;
+  wchar_t *textBuffer;
 
-  unsigned char attributes[count];
-  unsigned char *a = attributes;
+  if ((textBuffer = malloc(ARRAY_SIZE(textBuffer, count)))) {
+    unsigned char *attributesBuffer;
 
-  {
-    unsigned int i;
+    if ((attributesBuffer = malloc(ARRAY_SIZE(attributesBuffer, count)))) {
+      wchar_t *text = textBuffer;
+      unsigned char *attributes = attributesBuffer;
 
-    for (i=0; i<count; i+=1) {
-      const ScreenCharacter *character = &characters[i];
-      *t++ = character->text;
-      *a++ = character->attributes;
+      const ScreenCharacter *character = characters;
+      const ScreenCharacter *end = character + count;
+
+      while (character < end) {
+        *text++ = character->text;
+        *attributes++ = character->attributes;
+        character += 1;
+      }
+
+      sayWideCharacters(&spk, textBuffer, attributesBuffer, count, options);
+      free(attributesBuffer);
+    } else {
+      logMallocError();
     }
-  }
 
-  sayWideCharacters(&spk, text, attributes, count, options);
+    free(textBuffer);
+  } else {
+    logMallocError();
+  }
 }
 
 void
@@ -1056,6 +1072,7 @@ speakCharacters (const ScreenCharacter *characters, size_t count, int spell, int
     }
   } else if (count == 1) {
     wchar_t character = characters[0].text;
+    unsigned char attributes = characters[0].attributes;
     const char *prefix = NULL;
 
     if (iswupper(character)) {
@@ -1074,31 +1091,47 @@ speakCharacters (const ScreenCharacter *characters, size_t count, int spell, int
           sayOptions |= SAY_OPT_HIGHER_PITCH;
           break;
       }
+    } else if (iswpunct(character)) {
+      sayOptions |= SAY_OPT_ALL_PUNCTUATION;
     }
 
     if (prefix) {
-      wchar_t buffer[0X100];
-      size_t length = makeWcharsFromUtf8(prefix, buffer, ARRAY_COUNT(buffer));
+      wchar_t textBuffer[0X100];
+      size_t length = makeWcharsFromUtf8(prefix, textBuffer, ARRAY_COUNT(textBuffer));
 
-      buffer[length++] = WC_C(' ');
-      buffer[length++] = character;
-      sayWideCharacters(&spk, buffer, NULL, length, sayOptions);
+      textBuffer[length++] = WC_C(' ');
+      textBuffer[length++] = character;
+
+      unsigned char attributesBuffer[length];
+      memset(attributesBuffer, SCR_COLOUR_DEFAULT, length);
+      attributesBuffer[length-1] = attributes;
+
+      sayWideCharacters(&spk, textBuffer, attributesBuffer, length, sayOptions);
     } else {
-      if (iswpunct(character)) sayOptions |= SAY_OPT_ALL_PUNCTUATION;
-      sayWideCharacters(&spk, &character, NULL, 1, sayOptions);
+      sayWideCharacters(&spk, &character, &attributes, 1, sayOptions);
     }
   } else if (spell) {
-    wchar_t string[count * 2];
-    size_t length = 0;
-    unsigned int index = 0;
+    size_t length = count * 2;
+    wchar_t textBuffer[length];
+    unsigned char attributesBuffer[length];
 
-    while (index < count) {
-      string[length++] = characters[index++].text;
-      string[length++] = WC_C(' ');
+    wchar_t *text = textBuffer;
+    unsigned char *attributes = attributesBuffer;
+
+    const ScreenCharacter *character = characters;
+    const ScreenCharacter *end = character + count;
+
+    while (character < end) {
+      *text++ = character->text;
+      *attributes++ = character->attributes;
+
+      *text++ = WC_C(' ');
+      *attributes++ = SCR_COLOUR_DEFAULT;
+
+      character += 1;
     }
 
-    string[length] = WC_C('\0');
-    sayWideCharacters(&spk, string, NULL, length, sayOptions);
+    sayWideCharacters(&spk, textBuffer, attributesBuffer, length-1, sayOptions);
   } else {
     sayScreenCharacters(characters, count, sayOptions);
   }
@@ -1141,22 +1174,6 @@ isContracting (void) {
   return isContractedBraille() && contractionTable;
 }
 
-int
-getUncontractedCursorOffset (int x, int y) {
-  return ((y == ses->winy) && (x >= ses->winx) && (x < scr.cols))?
-         (x - ses->winx):
-         BRL_NO_CURSOR;
-}
-
-int
-getContractedCursor (void) {
-  int offset = getUncontractedCursorOffset(scr.posx, scr.posy);
-
-  return ((offset != BRL_NO_CURSOR) && !ses->hideScreenCursor)?
-         offset:
-         CTB_NO_CURSOR;
-}
-
 int
 getContractedLength (unsigned int outputLimit) {
   int inputLength = scr.cols - ses->winx;
@@ -1170,10 +1187,10 @@ getContractedLength (unsigned int outputLimit) {
   int outputOffsets[offsetCount + 1];
 
   contractText(
-    contractionTable,
+    contractionTable, NULL,
     inputBuffer, &inputLength,
     outputBuffer, &outputLength,
-    outputOffsets, getContractedCursor()
+    outputOffsets, getCursorOffsetForContracting()
   );
 
   for (int length=0; length<inputLength; length+=1) {
@@ -1409,7 +1426,7 @@ showDotPattern (unsigned char dots, unsigned char duration) {
   }
 
   memset(brl.buffer, dots, brl.textColumns*brl.textRows);
-  if (!writeBrailleWindow(&brl, NULL, 0)) return 0;
+  if (!writeBrailleWindow(&brl, NULL, SCQ_GOOD)) return 0;
 
   drainBrailleOutput(&brl, duration);
   return 1;
@@ -1461,7 +1478,7 @@ typedef struct {
 
   struct {
     AsyncEvent *event;
-    unsigned finished:1;
+    unsigned char finished:1;
   } wait;
 } CoreTaskData;
 
@@ -1634,8 +1651,11 @@ brlttyConstruct (int argc, char *argv[]) {
 
 int
 brlttyDestruct (void) {
+  if (prefs.saveOnExit) savePreferences();
+
   suspendUpdates();
   stopCoreTasks();
+
   endProgram();
   endCommandQueue();
   return 1;
