@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -67,7 +67,7 @@ findLineBreakOpportunities (
    * 9  digits
    */
 
-  while (lbo->index < end) {
+  while (lbo->index <= end) {
     unsigned char *opportunity = &opportunities[lbo->index];
 
     lbo->previous = lbo->before;
@@ -436,8 +436,6 @@ findLineBreakOpportunities (
 }
 
 #else /* HAVE_ICU */
-#include "ascii.h"
-
 typedef struct {
   unsigned int index;
   int wasSpace;
@@ -456,7 +454,7 @@ findLineBreakOpportunities (
   unsigned char *opportunities,
   const wchar_t *characters, unsigned int end
 ) {
-  while (lbo->index < end) {
+  while (lbo->index <= end) {
     int isSpace = testCharacter(bcd, characters[lbo->index], CTC_Space);
     opportunities[lbo->index] = lbo->wasSpace && !isSpace;
 
@@ -466,6 +464,19 @@ findLineBreakOpportunities (
 }
 #endif /* HAVE_ICU */
 
+static int
+isLineBreakOpportunity (
+  BrailleContractionData *bcd,
+  LineBreakOpportunitiesState *lbo,
+  unsigned char *opportunities
+) {
+  unsigned int index = getInputConsumed(bcd);
+  if (index == getInputCount(bcd)) return 1;
+
+  findLineBreakOpportunities(bcd, lbo, opportunities, bcd->input.begin, index);
+  return opportunities[index];
+}
+
 static inline ContractionTableHeader *
 getContractionTableHeader (BrailleContractionData *bcd) {
   return bcd->table->data.internal.header.fields;
@@ -536,6 +547,8 @@ makeDecomposedBraille (BrailleContractionData *bcd, wchar_t character, BYTE *cel
       if (character != entry->value) break;
 
       const ContractionTableRule *rule = entry->always;
+      if (!rule) break;
+
       unsigned int cellCount = rule->replen;
       if (!cellCount) break;
       if ((end - from) < cellCount) break;
@@ -986,7 +999,7 @@ contractText_native (BrailleContractionData *bcd) {
   BYTE *destjoin = NULL;
   BYTE *destlast = NULL;
 
-  unsigned char lineBreakOpportunities[getInputCount(bcd)];
+  unsigned char lineBreakOpportunities[getInputCount(bcd) + 1];
   LineBreakOpportunitiesState lbo;
   prepareLineBreakOpportunitiesState(&lbo);
 
@@ -1037,8 +1050,10 @@ contractText_native (BrailleContractionData *bcd) {
         int outputLength = bcd->output.end - bcd->output.current;
 
         contractText(
-          bcd->table, inputBuffer, &inputLength,
-          bcd->output.current, &outputLength, NULL, CTB_NO_CURSOR
+          bcd->table, NULL,
+          inputBuffer, &inputLength,
+          bcd->output.current, &outputLength,
+          NULL, CTB_NO_CURSOR
         );
 
         bcd->output.current += outputLength;
@@ -1172,8 +1187,7 @@ contractText_native (BrailleContractionData *bcd) {
       bcd->input.current += 1;
     }
 
-    findLineBreakOpportunities(bcd, &lbo, lineBreakOpportunities, bcd->input.begin, getInputConsumed(bcd));
-    if (lineBreakOpportunities[getInputConsumed(bcd)]) {
+    if (isLineBreakOpportunity(bcd, &lbo, lineBreakOpportunities)) {
       srcjoin = bcd->input.current;
       destjoin = bcd->output.current;
 
