@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -324,7 +324,7 @@ ASYNC_CONDITION_TESTER(testSpeechResponseReceived) {
 
 static int
 awaitSpeechResponse (SpeechDriverThread *sdt, int timeout) {
-  return asyncAwaitCondition(timeout, testSpeechResponseReceived, (void *)sdt);
+  return asyncAwaitCondition(timeout, testSpeechResponseReceived, sdt);
 }
 
 static void sendSpeechRequest (SpeechDriverThread *sdt);
@@ -458,15 +458,14 @@ handleSpeechRequest (SpeechDriverThread *sdt, SpeechRequest *req) {
     switch (req->type) {
       case REQ_SAY_TEXT: {
         SayOptions options = req->arguments.sayText.options;
+        if (options & SAY_OPT_MUTE_FIRST) speech->mute(spk);
+
         int restorePitch = 0;
         int restorePunctuation = 0;
 
-        if (options & SAY_OPT_MUTE_FIRST) speech->mute(spk);
-
         if (options & SAY_OPT_HIGHER_PITCH) {
           if (spk->setPitch) {
             unsigned char pitch = prefs.speechPitch + 7;
-
             if (pitch > SPK_PITCH_MAXIMUM) pitch = SPK_PITCH_MAXIMUM;
 
             if (pitch != prefs.speechPitch) {
@@ -830,10 +829,10 @@ THREAD_FUNCTION(runSpeechDriverThread) {
 
   setThreadState(sdt, THD_STARTING);
 
-  if ((sdt->requestEvent = asyncNewEvent(handleSpeechRequestEvent, (void *)sdt))) {
+  if ((sdt->requestEvent = asyncNewEvent(handleSpeechRequestEvent, sdt))) {
     if (startSpeechDriver(sdt)) {
       setThreadReady(sdt);
-      asyncWaitFor(testSpeechDriverThreadStopping, (void *)sdt);
+      asyncWaitFor(testSpeechDriverThreadStopping, sdt);
       stopSpeechDriver(sdt);
     } else {
       logMessage(LOG_CATEGORY(SPEECH_EVENTS), "driver construction failure");
@@ -872,7 +871,7 @@ constructSpeechDriverThread (
   SpeechDriverThread *sdt;
 
   if ((sdt = malloc(sizeof(*sdt)))) {
-    memset((void *)sdt, 0, sizeof(*sdt));
+    memset(sdt, 0, sizeof(*sdt));
     setThreadState(sdt, THD_CONSTRUCTING);
     setResponsePending(sdt);
 
@@ -883,11 +882,11 @@ constructSpeechDriverThread (
       spk->driver.thread = sdt;
 
 #ifdef GOT_PTHREADS
-      if ((sdt->messageEvent = asyncNewEvent(handleSpeechMessageEvent, (void *)sdt))) {
+      if ((sdt->messageEvent = asyncNewEvent(handleSpeechMessageEvent, sdt))) {
         pthread_t threadIdentifier;
         int createError = createThread("speech-driver",
                                        &threadIdentifier, NULL,
-                                       runSpeechDriverThread, (void *)sdt);
+                                       runSpeechDriverThread, sdt);
 
         if (!createError) {
           sdt->threadIdentifier = threadIdentifier;
@@ -924,7 +923,7 @@ constructSpeechDriverThread (
       deallocateQueue(sdt->requestQueue);
     }
 
-    free((void *)sdt);
+    free(sdt);
   } else {
     logMallocError();
   }
@@ -958,6 +957,6 @@ destroySpeechDriverThread (SpeechSynthesizer *spk) {
 
   sdt->speechSynthesizer->driver.thread = NULL;
   deallocateQueue(sdt->requestQueue);
-  free((void *)sdt);
+  free(sdt);
 }
 #endif /* ENABLE_SPEECH_SUPPORT */
